/****************************************************************************
 *  pcomndis.h    : $Revision:   1.10  $
 *  Modified      : $Date:   May 13 1996 14:48:46  $
 *  By            : $Author:   tmortsol  $, U.S. Robotics Access Corp.
 *****************************************************************************
 *          Copyright (c) $Date:   May 13 1996 14:48:46  $,                  *
 *            U.S. Robotics Access Corp., Microsoft Corp.                    *
 *                       All Rights Reserved                                 *
 *****************************************************************************
 *                          "PPTP MODULE"                                    *
 * This software may be used to implement the POINT to POINT TUNNELING       *
 * PROTOCOL as defined by the PPTP forum.  The PPTP forum consists of:       *
 * U.S. Robotics, Microsoft, Ascend Communications, 3Com/Primary Access,     *
 * Telematics.                                                               *
 *****************************************************************************
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain this entire header
 *    including the above copyright notice, this list of conditions and
 *    the following disclaimer.
 * 2. The name of U.S. Robotics may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission of U.S. Robotics.
 *
 * THIS SOFTWARE IS PROVIDED BY U.S. ROBOTICS ACESS CORP. ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL U.S. ROBOTICS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*END_HEADER*/

/*
 * pcomndis.h
 *
 * This file contains target specific definitions used by the PPTP protocol
 * when compiled as an NT NDISWAN driver
 *
 * This header file is included by the "pptpcomm.h" header file
 *
 * Author: Tim Mortsolf, U.S. Robotics Access Corp.
 */

#if !defined(_pcomndis_h)
#define _pcomndis_h

/*
 * Source file is compiling as a NT driver
 * Initial system include files
 */

#include <ntddk.h>
#include <tdikrnl.h>
#include <ndis.h>
#include <ndiswan.h>
#include <ndistapi.h>

#undef USE_NATIVE_ASSERT
#include "debug.h"

/*
 * Define socket structures and routines used internally that are not defined
 * by NT kernel header files
 */

struct in_addr {
  union {
    struct { UCHAR s_b1,s_b2,s_b3,s_b4; } S_un_b;
    struct { USHORT s_w1,s_w2; } S_un_w;
    ULONG S_addr;
  } S_un;
#define s_addr  S_un.S_addr
};
struct sockaddr_in {
        short   sin_family;
        USHORT	sin_port;
        struct  in_addr sin_addr;
        UCHAR   sin_zero[8];
};

#define htons(a) ((((a) & 0XFF00) >> 8) | (((a) & 0X00FF) << 8))
#define ntohs(a) (htons(a))
#define htonl(a) ((((a) & 0XFF000000) >> 24) | (((a) & 0X00FF0000) >> 8) | \
		  (((a) & 0X0000FF00) << 8)  | (((a) & 0X000000FF) << 24))
#define ntohl(a) (htonl(a))

#define INADDR_NONE (0XFFFFFFFF)

/*
 * Define min() since it is not defined by NT header files
 */

#define min(a,b) (((a) < (b)) ? (a) : (b))

/*
 * Define how to do output I/O
 */

#define PptpPrintf DBG_TRACE
#define PptpKdPrint DBG_TRACE

/*
 * #defines for how the product should compile
 */

#define PPTP_SERVER 1
#define PPTP_SYMMETRIC_SERVER 1

/*
 * #defines for default target specific parameters
 */

#define PPTP_DEFAULT_NUMBER_CALL_LINES (2)

/*
 * Define basic locking privitives
 */

typedef struct {
  KSPIN_LOCK spinLock;
  KIRQL oldIrql;
  BOOLEAN lockHeld;
} PptpLock;

#define PPTP_USE_STRONG_LOCKS
//#define PPTP_USE_NO_LOCKS

#ifndef PPTP_USE_NO_LOCKS

#define PPTP_LOCK_INIT(lock) \
  KeInitializeSpinLock(&lock.spinLock); lock.lockHeld = 0

#define PPTP_LOCK(lock) \
  KeAcquireSpinLock(&lock.spinLock, &lock.oldIrql); \
  ASSERT(!lock.lockHeld); \
  lock.lockHeld = 1

#define PPTP_UNLOCK(lock) \
  ASSERT(lock.lockHeld); \
  lock.lockHeld = 0; \
  KeReleaseSpinLock(&lock.spinLock, lock.oldIrql)

#endif // !PPTP_USE_NO_LOCKS

/*
 * Defined memory pool tag.
 */

#if defined(POOL_TAGGING)

#if defined(UNIQUE_POOL_TAGS)
extern ULONG makePptpTag(int lineNumber);
#define PPTP_MEMORYPOOL_TAG makePptpTag(__LINE__)
#else
#define PPTP_MEMORYPOOL_TAG 'PTPP'
#endif /* defined(UNIQUE_POOL_TAGS) */

#define NEW new(PPTP_MEMORYPOOL_TAG)

#else

#define NEW new

#endif /* POOL_TAGGING */

#endif /* !defined(_pcomndis_h) */

